/*============================================================================*/
/**
 * la_interface.h
 * Version: 1.0 (22.11.2010)
 * 
 * Funktionen zum Zugriff auf den USB-LOGI-500.
 * (Basiert auf der original DELIB von Deditec fr Linux)
 *
 * Bentigt die FDTI Treiber von http://www.ftdichip.com/Drivers/D2XX.htm
 *
 * TODO: Dokumentation, Fehlercodes und -meldungen verbessern
 **
/*============================================================================*/

#ifndef LA_INTERFACE_H
#define LA_INTERFACE_H

#include "ftdi/WinTypes.h"
#include "ftdi/ftd2xx.h"


/*============================================================================*/
/* If defined, additional debug messages are shown */
/*#define DEBUG*/


/*============================================================================*/
/* All available device commands */
#define CMD_PING	0x12
#define CMD_SEND_8_BYTE	0x23
#define CMD_GET_8_BYTE	0x34
#define CMD_SEND_N_BYTE	0x45
#define CMD_GET_N_BYTE	0x56

/*============================================================================*/
/* Error codes */
#define LA_ERR_NONE				0
#define LA_ERR_DEVICE_NOT_FOUND			-1
#define LA_ERR_COMMUNICATION_ERROR		-2
#define LA_ERR_ILLEGAL_HANDLE			-10
#define LA_ERR_FUNCTION_NOT_DEFINED		-11
#define LA_ERR_ILLEGAL_COM_HANDLE		-12

#define LA_ERR_DEVICE_NOT_SUPPORTED		-256
#define LA_ERR_ILLEGAL_ARGUMENTS		-257
#define LA_ERR_OUT_OF_MEMORY			-1023

/*============================================================================*/
/* Typedefs */

struct LA_Module
{
	FT_HANDLE ftHandle;
	unsigned char jobID;
};

typedef struct LA_Module *LA_Handle;

/*============================================================================*/
/* Function prototypes */


LA_Handle LA_OpenModule();
int LA_CloseModule(LA_Handle handle);
int LA_CheckHandle(LA_Handle handle);
int LA_FTDISendCommand(LA_Handle handle, unsigned char command, unsigned long address,
		unsigned char be, unsigned char *data, unsigned long data_length);
int LA_FTDISendCommandMultipleBytes(LA_Handle handle, unsigned char command, unsigned long address,
		unsigned long addressDepth, unsigned long wiederholungen, unsigned char *data, unsigned long data_length);



unsigned long LA_Ping(LA_Handle handle, unsigned long value);



void LA_WriteByte(LA_Handle handle, unsigned long address, unsigned long value);
void LA_WriteWord(LA_Handle handle, unsigned long address, unsigned long value);
void LA_WriteLong(LA_Handle handle, unsigned long address, unsigned long value);
void LA_WriteLongLong(LA_Handle handle, unsigned long address, unsigned long long value);

unsigned long LA_ReadByte(LA_Handle handle, unsigned long address);
unsigned long LA_ReadWord(LA_Handle handle, unsigned long address);
unsigned long LA_ReadLong(LA_Handle handle, unsigned long address);
unsigned long long LA_ReadLongLong(LA_Handle handle, unsigned long address);



int LA_ReadMultipleBytes(LA_Handle handle, unsigned long address, unsigned long addressDepth,
		unsigned long iterations, unsigned char *buffer, unsigned long bufferLength);
int LA_WriteMultipleBytes(LA_Handle handle, unsigned long address, unsigned long addressDepth,
		unsigned long iterations, unsigned char *buffer, unsigned long bufferLength);



void LA_SetError(int errorcode, int suberror, const char *errortext, const char *filename, unsigned line);
void LA_ClearLastError();
int LA_GetLastError();
const char *LA_GetLastErrorText();

#endif

