

#ifndef SAMPLER_H
#define SAMPLER_H

#include "la/la_interface.h"

/* 
Adressen der Register und Speicher vom USB-LOGI-500
Die Adressen werden für LA_ReadXXX/LA_WriteXXX benötigt.
*/
#define LA_ADDR_CFG0_L			0x0000
#define LA_ADDR_CFG0_H			0x0004
#define LA_ADDR_CFG1_L			0x0008
#define LA_ADDR_CFG1_H			0x000C
#define LA_ADDR_TRIGGER_A_0		0x0010
#define LA_ADDR_TRIGGER_A_1		0x0018
#define LA_ADDR_TRIGGER_A_RE		0x0020
#define LA_ADDR_TRIGGER_A_FE		0x0028
#define LA_ADDR_TRIGGER_B_0		0x0030
#define LA_ADDR_TRIGGER_B_1		0x0038
#define LA_ADDR_TRIGGER_B_RE		0x0040
#define LA_ADDR_TRIGGER_B_FE		0x0048
#define LA_ADDR_TRIGGER_HELPER_0	0x0050
#define LA_ADDR_TRIGGER_HELPER_1	0x0051
#define LA_ADDR_TRIGGER_SELECTION	0x0052
#define LA_ADDR_TRIGGER_START		0x0053
#define LA_ADDR_TRIGGER_COUNTER_0	0x0058
#define LA_ADDR_CH_IN_L			0x00E0
#define LA_ADDR_CH_IN_H			0x00E4
#define LA_ADDR_STATUS			0x00E8
#define LA_ADDR_EXT_RAM_STATUS		0x00E9
#define LA_ADDR_TRIGGER_CNT		0x00D0
#define LA_ADDR_ADDR_CNT_TRIGGER	0x00D8
#define LA_ADDR_ADDR_CNT		0x00F0
#define LA_ADDR_RAM0			0x6000
#define LA_ADDR_RAM1			0x8000
#define LA_ADDR_RAM2			0xA000
#define LA_ADDR_RAM3			0xC000
#define LA_ADDR_DEVICE_VERSION		0xFFF4

/*
Bitnummern der einzelnen Bits im Konfigurationsregister 0 (LA_ADDR_CFG0_L)
*/
#define LA_BIT_START_SAMPLING		0
#define LA_BIT_STOP_SAMPLING		1
#define LA_BIT_PC_MODE			2
#define LA_BIT_TRIGGER_IGNORE		3
#define LA_BIT_PRETRIGGER		4
#define LA_BIT_INTERNAL_RAM		7
#define LA_BIT_MODE_500MS		8
#define LA_BIT_MODE_250MS		9
#define LA_BIT_ADDR_CNT_RESET		10
#define LA_BIT_MEM_AREA			11

#define LA_BIT_CLK_DIVIDER		8

/*
Statusnummern des LA
*/
#define LA_STATUS_READY			0
#define LA_STATUS_FILLING_PRETRIGGER	2
#define LA_STATUS_WAITING_FOR_TRIGGER	3
#define LA_STATUS_SAMPLING		5
#define LA_STATUS_SAMPLING_DONE		6

/* Hardwarespezifikation */
#define LA_NUM_CHANNELS			36
#define LA_NUM_SAMPLES			4096
#define LA_RAM_ADDR_MASK		0xFFF
#define LA_SAMPLE_CLK			150000000


struct la_trigger
{
	unsigned char		low;
	unsigned char		high;
	unsigned char		rising_edge;
	unsigned char		falling_edge;
};

struct la_signal
{
	char			*name;
	unsigned char		num_channels;
	unsigned char		*channels;
};

struct la_config
{
	unsigned long 		sample_rate;
	unsigned char		pretrigger;
	unsigned char		trigger_ignore;
	
	unsigned char		num_signals;
	struct la_signal	*signals;

	struct la_trigger	triggerA[36];
	struct la_trigger	triggerB[36];
	unsigned char		trigger_helper0;
	unsigned char		trigger_helper1;
	unsigned char		trigger_selection;
	unsigned char		trigger_start_trigger;
	unsigned long		trigger_counter;
};



int la_connect();
int la_disconnect();
void la_build_trigger_reg_values(struct la_trigger *trigger, unsigned long long *reg_values);
int la_configure(struct la_config *cfg);
int la_start_sampling();
int la_stop_sampling();
int la_get_status(int *progress);
unsigned long la_get_addr_cnt();
unsigned long la_get_trigger_cnt();
int la_get_sampling_data(unsigned long long *data, unsigned long length);
void la_test_error();




#endif

