
#include <string.h>

#include <libxml/parser.h>
#include <libxml/xpath.h>

#include "xml.h"


/* Globales Handle zum XML Dokument */
xmlDocPtr doc = NULL;


/*******************************************************************************
XML Datei zur Bearbeitung öffnen

Rückgabewerte:
 0 = Alles OK
-1 = Datei konnte nicht geöffnet werden
********************************************************************************/
int xml_open(const char *filename)
{
	xmlInitParser();
	doc = xmlParseFile(filename);

	if (doc == NULL)
	{
		return -1;
	}
	return 0;
}

/*******************************************************************************
XML Datei schließen

Rückgabewerte:
 0 = Alles OK

********************************************************************************/
int xml_close()
{
	xmlFreeDoc(doc); 
	xmlCleanupParser();

	doc = NULL;

	return 0;
}

/*******************************************************************************
Werte aus einer XML Datei mittels XPath ermitteln.

Rückgabewerte:
 0 = Alles OK
-1 = Kein Dokument geöffnet
-2 = Anfrage lieferte keine Ergebnisse
-3 = String Buffer zu klein
********************************************************************************/
int xml_get_string(const char *xpath, char *str, unsigned length)
{
	char *content;
	xmlXPathContextPtr xpathCtx; 
	xmlXPathObjectPtr xpathObj;

	/* Kein Dokument geöffnet */
	if (doc == NULL)
	{
		return -1;
	}

	/* XPath Context erstellen */
	xpathCtx = xmlXPathNewContext(doc);
	if (xpathCtx == NULL)
	{
		return -1;
	}

	/* XPath Query auswerten */
	xpathObj = xmlXPathEvalExpression(xpath, xpathCtx);
	
	if (xpathObj == NULL || xpathObj->nodesetval == NULL || xpathObj->nodesetval->nodeNr < 1)
	{
		/* Keine Ergebnisse */
		xmlXPathFreeContext(xpathCtx);
		return -2;
	}

	/* String kopieren, wenn genug Platz im übergebenen Buffer */
	content = xmlXPathCastNodeToString(xpathObj->nodesetval->nodeTab[0]);
	if (length < (strlen(content) + 1))
	{
		return -3;
	}
	strcpy(str, content); 

	/* Resourcen freigeben */
	xmlXPathFreeObject(xpathObj);
	xmlXPathFreeContext(xpathCtx);

	return 0;
}

/*******************************************************************************
Anzahl der Knoten einer XPath Abfrage ermitteln

Rückgabewerte:
>= 0 = Alles OK
-1 = Kein Dokument geöffnet
-2 = Anfrage lieferte keine Ergebnisse
********************************************************************************/
int xml_get_element_count(const char *xpath)
{
	int count;
	xmlXPathContextPtr xpathCtx; 
	xmlXPathObjectPtr xpathObj;

	/* Kein Dokument geöffnet */
	if (doc == NULL)
	{
		return -1;
	}

	/* XPath Context erstellen */
	xpathCtx = xmlXPathNewContext(doc);
	if (xpathCtx == NULL)
	{
		return -1;
	}

	/* XPath Query auswerten */
	xpathObj = xmlXPathEvalExpression(xpath, xpathCtx);

	if (xpathObj == NULL || xpathObj->nodesetval == NULL)
	{
		/* Keine Ergebnisse */
		xmlXPathFreeContext(xpathCtx);
		return -2;
	}

	count = xpathObj->nodesetval->nodeNr;

	/* Resourcen freigeben */
	xmlXPathFreeObject(xpathObj);
	xmlXPathFreeContext(xpathCtx);

	/* Anzahl Elemente zurückgeben */
	return count;
}

